@@ -13,35 +13,32 @@ module Language.Haskell.GhcMod (
   , ModuleString
   , Expression
   -- * 'IO' utilities
+  , bootInfo
   , browseModule
   , checkSyntax
   , lintSyntax
+  , expandTemplate
   , infoExpr
   , typeExpr
   , listModules
   , listLanguages
   , listFlags
   , debugInfo
-  -- * Converting the 'Ghc' monad to the 'IO' monad
-  , withGHC
-  , withGHCDummyFile
-  -- * 'Ghc' utilities
-  , browse
-  , check
-  , info
-  , typeOf
-  , listMods
-  , debug
+  , rootInfo
+  , packageDoc
+  , findSymbol
   ) where
 
+import Language.Haskell.GhcMod.Boot
 import Language.Haskell.GhcMod.Browse
 import Language.Haskell.GhcMod.Check
 import Language.Haskell.GhcMod.Cradle
 import Language.Haskell.GhcMod.Debug
+import Language.Haskell.GhcMod.Find
 import Language.Haskell.GhcMod.Flag
-import Language.Haskell.GhcMod.GHCApi
 import Language.Haskell.GhcMod.Info
 import Language.Haskell.GhcMod.Lang
 import Language.Haskell.GhcMod.Lint
 import Language.Haskell.GhcMod.List
+import Language.Haskell.GhcMod.PkgDoc
 import Language.Haskell.GhcMod.Types
