@@ -5,16 +5,19 @@ module Language.Haskell.GhcMod.Gap (
   , mkTarget
   , withStyle
   , setLogAction
-  , supportedExtensions
   , getSrcSpan
   , getSrcFile
-  , setCtx
+  , withContext
   , fOptions
   , toStringBuffer
-  , liftIO
   , showSeverityCaption
   , setCabalPkg
-  , addDevPkgs
+  , setHideAllPackages
+  , addPackageFlags
+  , setDeferTypeErrors
+  , setWarnTypedHoles
+  , setDumpSplices
+  , isDumpSplices
   , filterOutChildren
   , infoThing
   , pprInfo
@@ -22,17 +25,21 @@ module Language.Haskell.GhcMod.Gap (
   , errorMsgSpan
   , typeForUser
   , deSugar
-#if __GLASGOW_HASKELL__ >= 702
-#else
-  , module Pretty
-#endif
+  , showDocWith
+  , GapThing(..)
+  , fromTyThing
+  , fileModSummary
+  , WarnFlags
+  , emptyWarnFlags
   ) where
 
 import Control.Applicative hiding (empty)
-import Control.Monad
-import Data.List
-import Data.Maybe
-import Data.Time.Clock
+import Control.Monad (filterM)
+import CoreSyn (CoreExpr)
+import Data.List (intersperse)
+import Data.Maybe (catMaybes)
+import Data.Time.Clock (UTCTime)
+import DataCon (dataConRepType)
 import Desugar (deSugarExpr)
 import DynFlags
 import ErrUtils
@@ -45,35 +52,32 @@ import Outputable
 import PprTyThing
 import StringBuffer
 import TcType
-import TcRnTypes
-import CoreSyn
+import Var (varType)
 
 import qualified InstEnv
 import qualified Pretty
 import qualified StringBuffer as SB
-#if __GLASGOW_HASKELL__ >= 707
+
+#if __GLASGOW_HASKELL__ >= 708
 import FamInstEnv
+import ConLike (ConLike(..))
+import PatSyn (patSynType)
+#else
+import TcRnTypes
 #endif
 
 #if __GLASGOW_HASKELL__ >= 706
 import GHC hiding (ClsInst)
 #else
 import GHC hiding (Instance)
-#endif
-
-#if __GLASGOW_HASKELL__ >= 702
-import CoreMonad (liftIO)
-#else
-import HscTypes (liftIO)
-import Pretty
-#endif
-
-
-#if __GLASGOW_HASKELL__ < 706
 import Control.Arrow hiding ((<+>))
 import Data.Convertible
 #endif
 
+#if __GLASGOW_HASKELL__ >= 704
+import qualified Data.IntSet as I (IntSet, empty)
+#endif
+
 ----------------------------------------------------------------
 ----------------------------------------------------------------
 --
@@ -110,14 +114,13 @@ setLogAction df f =
     df { log_action = f df }
 #endif
 
-----------------------------------------------------------------
-----------------------------------------------------------------
-
-supportedExtensions :: [String]
-#if __GLASGOW_HASKELL__ >= 700
-supportedExtensions = supportedLanguagesAndExtensions
+showDocWith :: DynFlags -> Pretty.Mode -> Pretty.Doc -> String
+#if __GLASGOW_HASKELL__ >= 708
+-- Pretty.showDocWith disappeard.
+-- https://github.com/ghc/ghc/commit/08a3536e4246e323fbcd8040e0b80001950fe9bc
+showDocWith dflags mode = Pretty.showDoc mode (pprCols dflags)
 #else
-supportedExtensions = supportedLanguages
+showDocWith _ = Pretty.showDocWith
 #endif
 
 ----------------------------------------------------------------
@@ -159,38 +162,50 @@ fOptions :: [String]
 fOptions = [option | (option,_,_) <- fFlags]
         ++ [option | (option,_,_) <- fWarningFlags]
         ++ [option | (option,_,_) <- fLangFlags]
-#elif __GLASGOW_HASKELL__ == 702
+#else
 fOptions = [option | (option,_,_,_) <- fFlags]
-#else
-fOptions = [option | (option,_,_) <- fFlags]
+        ++ [option | (option,_,_,_) <- fWarningFlags]
+        ++ [option | (option,_,_,_) <- fLangFlags]
 #endif
 
 ----------------------------------------------------------------
 ----------------------------------------------------------------
 
-setCtx :: [ModSummary] -> Ghc Bool
-#if __GLASGOW_HASKELL__ >= 704
-setCtx ms = do
-#if __GLASGOW_HASKELL__ >= 706
-    let modName = IIModule . moduleName . ms_mod
-#else
-    let modName = IIModule . ms_mod
-#endif
-    top <- map modName <$> filterM isTop ms
-    setContext top
-    return (not . null $ top)
-#else
-setCtx ms = do
-    top <- map ms_mod <$> filterM isTop ms
-    setContext top []
-    return (not . null $ top)
-#endif
+fileModSummary :: FilePath -> Ghc ModSummary
+fileModSummary file = do
+    mss <- getModuleGraph
+    let [ms] = filter (\m -> ml_hs_file (ms_location m) == Just file) mss
+    return ms
+
+withContext :: Ghc a -> Ghc a
+withContext action = gbracket setup teardown body
   where
-    isTop mos = lookupMod ||> returnFalse
-      where
-        lookupMod = lookupModule (ms_mod_name mos) Nothing >> return True
-        returnFalse = return False
-
+    setup = getContext
+    teardown = setCtx
+    body _ = do
+        topImports >>= setCtx
+        action
+    topImports = do
+        mss <- getModuleGraph
+        ms <- map modName <$> filterM isTop mss
+#if __GLASGOW_HASKELL__ >= 704
+        return ms
+#else
+        return (ms,[])
+#endif
+    isTop mos = lookupMod mos ||> returnFalse
+    lookupMod mos = lookupModule (ms_mod_name mos) Nothing >> return True
+    returnFalse = return False
+#if __GLASGOW_HASKELL__ >= 706
+    modName = IIModule . moduleName . ms_mod
+    setCtx = setContext
+#elif __GLASGOW_HASKELL__ >= 704
+    modName = IIModule . ms_mod
+    setCtx = setContext
+#else
+    modName = ms_mod
+    setCtx = uncurry setContext
+#endif
 
 showSeverityCaption :: Severity -> String
 #if __GLASGOW_HASKELL__ >= 706
@@ -204,7 +219,7 @@ showSeverityCaption = const ""
 ----------------------------------------------------------------
 
 setCabalPkg :: DynFlags -> DynFlags
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
 setCabalPkg dflag = gopt_set dflag Opt_BuildingCabalPackage
 #else
 setCabalPkg dflag = dopt_set dflag Opt_BuildingCabalPackage
@@ -212,18 +227,45 @@ setCabalPkg dflag = dopt_set dflag Opt_BuildingCabalPackage
 
 ----------------------------------------------------------------
 
-addDevPkgs :: DynFlags -> [Package] -> DynFlags
-addDevPkgs df []   = df
-addDevPkgs df pkgs = df''
-  where
-#if __GLASGOW_HASKELL__ >= 707
-    df' = gopt_set df Opt_HideAllPackages
+setHideAllPackages :: DynFlags -> DynFlags
+#if __GLASGOW_HASKELL__ >= 708
+setHideAllPackages df = gopt_set df Opt_HideAllPackages
 #else
-    df' = dopt_set df Opt_HideAllPackages
+setHideAllPackages df = dopt_set df Opt_HideAllPackages
+#endif
+
+addPackageFlags :: [Package] -> DynFlags -> DynFlags
+addPackageFlags pkgs df =
+    df { packageFlags = packageFlags df ++ expose `map` pkgs }
+  where
+    expose pkg = ExposePackageId $ showPkgId pkg
+
+----------------------------------------------------------------
+
+setDumpSplices :: DynFlags -> DynFlags
+setDumpSplices dflag = dopt_set dflag Opt_D_dump_splices
+
+isDumpSplices :: DynFlags -> Bool
+isDumpSplices dflag = dopt Opt_D_dump_splices dflag
+
+----------------------------------------------------------------
+
+
+setDeferTypeErrors :: DynFlags -> DynFlags
+#if __GLASGOW_HASKELL__ >= 708
+setDeferTypeErrors dflag = gopt_set dflag Opt_DeferTypeErrors
+#elif __GLASGOW_HASKELL__ >= 706
+setDeferTypeErrors dflag = dopt_set dflag Opt_DeferTypeErrors
+#else
+setDeferTypeErrors = id
+#endif
+
+setWarnTypedHoles :: DynFlags -> DynFlags
+#if __GLASGOW_HASKELL__ >= 708
+setWarnTypedHoles dflag = wopt_set dflag Opt_WarnTypedHoles
+#else
+setWarnTypedHoles = id
 #endif
-    df'' = df' {
-        packageFlags = map ExposePackage pkgs ++ packageFlags df
-      }
 
 ----------------------------------------------------------------
 ----------------------------------------------------------------
@@ -233,7 +275,7 @@ class HasType a where
 
 
 instance HasType (LHsBind Id) where
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
     getType _ (L spn FunBind{fun_matches = MG _ in_tys out_typ}) = return $ Just (spn, typ)
       where typ = mkFunTys in_tys out_typ
 #else
@@ -254,7 +296,7 @@ filterOutChildren get_thing xs
 infoThing :: String -> Ghc SDoc
 infoThing str = do
     names <- parseName str
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
     mb_stuffs <- mapM (getInfo False) names
     let filtered = filterOutChildren (\(t,_f,_i,_fam) -> t) (catMaybes mb_stuffs)
 #else
@@ -263,7 +305,7 @@ infoThing str = do
 #endif
     return $ vcat (intersperse (text "") $ map (pprInfo False) filtered)
 
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
 pprInfo :: Bool -> (TyThing, GHC.Fixity, [ClsInst], [FamInst]) -> SDoc
 pprInfo _ (thing, fixity, insts, famInsts)
     = pprTyThingInContextLoc thing
@@ -290,29 +332,23 @@ pprInfo pefas (thing, fixity, insts)
 ----------------------------------------------------------------
 
 errorMsgSpan :: ErrMsg -> SrcSpan
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
 errorMsgSpan = errMsgSpan
 #else
 errorMsgSpan = head . errMsgSpans
 #endif
 
 typeForUser :: Type -> SDoc
-#if __GLASGOW_HASKELL__ >= 707
+#if __GLASGOW_HASKELL__ >= 708
 typeForUser = pprTypeForUser
 #else
 typeForUser = pprTypeForUser False
 #endif
 
 deSugar :: TypecheckedModule -> LHsExpr Id -> HscEnv
-         -> IO (Maybe CoreSyn.CoreExpr)
-#if __GLASGOW_HASKELL__ >= 707
-deSugar tcm e hs_env = snd <$> deSugarExpr hs_env modu rn_env ty_env fi_env e
-  where
-    modu = ms_mod $ pm_mod_summary $ tm_parsed_module tcm
-    tcgEnv = fst $ tm_internals_ tcm
-    rn_env = tcg_rdr_env tcgEnv
-    ty_env = tcg_type_env tcgEnv
-    fi_env = tcg_fam_inst_env tcgEnv
+         -> IO (Maybe CoreExpr)
+#if __GLASGOW_HASKELL__ >= 708
+deSugar _   e hs_env = snd <$> deSugarExpr hs_env e
 #else
 deSugar tcm e hs_env = snd <$> deSugarExpr hs_env modu rn_env ty_env e
   where
@@ -321,3 +357,32 @@ deSugar tcm e hs_env = snd <$> deSugarExpr hs_env modu rn_env ty_env e
     rn_env = tcg_rdr_env tcgEnv
     ty_env = tcg_type_env tcgEnv
 #endif
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+data GapThing = GtA Type | GtT TyCon | GtN
+
+fromTyThing :: TyThing -> GapThing
+fromTyThing (AnId i)                   = GtA $ varType i
+#if __GLASGOW_HASKELL__ >= 708
+fromTyThing (AConLike (RealDataCon d)) = GtA $ dataConRepType d
+fromTyThing (AConLike (PatSynCon p))   = GtA $ patSynType p
+#else
+fromTyThing (ADataCon d)               = GtA $ dataConRepType d
+#endif
+fromTyThing (ATyCon t)                 = GtT t
+fromTyThing _                          = GtN
+
+----------------------------------------------------------------
+----------------------------------------------------------------
+
+#if __GLASGOW_HASKELL__ >= 704
+type WarnFlags = I.IntSet
+emptyWarnFlags :: WarnFlags
+emptyWarnFlags = I.empty
+#else
+type WarnFlags = [WarningFlag]
+emptyWarnFlags :: WarnFlags
+emptyWarnFlags = []
+#endif
