@@ -2,9 +2,11 @@
 
 module Language.Haskell.GhcMod.Internal (
   -- * Types
-    LogReader
-  , GHCOption
+    GHCOption
   , Package
+  , PackageBaseName
+  , PackageVersion
+  , PackageId
   , IncludeDir
   , CompilerOptions(..)
   -- * Cabal API
@@ -14,16 +16,17 @@ module Language.Haskell.GhcMod.Internal (
   , cabalDependPackages
   , cabalSourceDirs
   , cabalAllTargets
-  -- * GHC API
-  , canCheckFast
-  -- * Getting 'DynFlags'
+  -- * IO
+  , getSystemLibDir
   , getDynamicFlags
   -- * Initializing 'DynFlags'
-  , initializeFlags
   , initializeFlagsWithCradle
-  -- * 'GhcMonad'
+  -- * Targets
   , setTargetFiles
-  , checkSlowAndSet
+  -- * Logging
+  , withLogger
+  , setNoWaringFlags
+  , setAllWaringFlags
   -- * 'Ghc' Choice
   , (||>)
   , goNext
@@ -33,7 +36,7 @@ module Language.Haskell.GhcMod.Internal (
   ) where
 
 import Language.Haskell.GhcMod.CabalApi
-import Language.Haskell.GhcMod.ErrMsg
 import Language.Haskell.GhcMod.GHCApi
 import Language.Haskell.GhcMod.GHCChoice
+import Language.Haskell.GhcMod.Logger
 import Language.Haskell.GhcMod.Types
