@@ -1,9 +1,9 @@
 module Language.Haskell.GhcMod.Lang where
 
-import qualified Language.Haskell.GhcMod.Gap as Gap
+import DynFlags (supportedLanguagesAndExtensions)
 import Language.Haskell.GhcMod.Types
 
 -- | Listing language extensions.
 
 listLanguages :: Options -> IO String
-listLanguages opt = return $ convert opt Gap.supportedExtensions
+listLanguages opt = return $ convert opt supportedLanguagesAndExtensions
