@@ -16,7 +16,7 @@
 ;;;
 
 (defvar ghc-idle-timer-interval 30
- "*Period of idele timer in second. When timeout, the names of
+ "*Period of idle timer in second. When timeout, the names of
 unloaded modules are loaded")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -30,6 +30,8 @@ unloaded modules are loaded")
 ;; must be sorted
 (defconst ghc-reserved-keyword '("case" "deriving" "do" "else" "if" "in" "let" "module" "of" "then" "where"))
 
+(defconst ghc-extra-keywords '("ByteString" "Text"))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Local Variables
@@ -49,13 +51,13 @@ unloaded modules are loaded")
 (defvar ghc-merged-keyword nil) ;; completion for type/func/...
 (defvar ghc-language-extensions nil)
 (defvar ghc-option-flags nil)
-(defvar ghc-pragma-names '("LANGUAGE" "OPTIONS_GHC"))
+(defvar ghc-pragma-names '("LANGUAGE" "OPTIONS_GHC" "INCLUDE" "WARNING" "DEPRECATED" "INLINE" "NOINLINE" "ANN" "LINE" "RULES" "SPECIALIZE" "UNPACK" "SOURCE"))
 
 (defconst ghc-keyword-prefix "ghc-keyword-")
 (defvar ghc-keyword-Prelude nil)
 (defvar ghc-keyword-Control.Applicative nil)
-(defvar ghc-keyword-Control.Monad nil)
 (defvar ghc-keyword-Control.Exception nil)
+(defvar ghc-keyword-Control.Monad nil)
 (defvar ghc-keyword-Data.Char nil)
 (defvar ghc-keyword-Data.List nil)
 (defvar ghc-keyword-Data.Maybe nil)
@@ -64,14 +66,14 @@ unloaded modules are loaded")
 (defvar ghc-loaded-module nil)
 
 (defun ghc-comp-init ()
-  (add-hook 'find-file-hook 'ghc-import-module)
   (let* ((syms '(ghc-module-names
 		 ghc-language-extensions
 		 ghc-option-flags
+		 ;; hard coded in GHCMod.hs
 		 ghc-keyword-Prelude
 		 ghc-keyword-Control.Applicative
-		 ghc-keyword-Control.Monad
 		 ghc-keyword-Control.Exception
+		 ghc-keyword-Control.Monad
 		 ghc-keyword-Data.Char
 		 ghc-keyword-Data.List
 		 ghc-keyword-Data.Maybe
@@ -80,15 +82,15 @@ unloaded modules are loaded")
     (ghc-set syms vals))
   (ghc-add ghc-module-names "qualified")
   (ghc-add ghc-module-names "hiding")
+  ;; hard coded in GHCMod.hs
   (ghc-merge-keywords '("Prelude"
 			"Control.Applicative"
-			"Control.Monad"
 			"Control.Exception"
+			"Control.Monad"
 			"Data.Char"
 			"Data.List"
 			"Data.Maybe"
-			"System.IO"))
-  (run-with-idle-timer ghc-idle-timer-interval 'repeat 'ghc-idle-timer))
+			"System.IO")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -96,25 +98,22 @@ unloaded modules are loaded")
 ;;;
 
 (defun ghc-boot (n)
-  (if (not (executable-find ghc-module-command))
-      (message "%s not found" ghc-module-command)
-    (ghc-read-lisp-list
-     (lambda ()
-       (message "Initializing...")
-       (call-process ghc-module-command nil t nil "-l" "boot")
-       (message "Initializing...done"))
-     n)))
+  (prog2
+      (message "Initializing...")
+      (ghc-sync-process "boot\n" n)
+    (message "Initializing...done")))
 
 (defun ghc-load-modules (mods)
-  (if (not (executable-find ghc-module-command))
-      (message "%s not found" ghc-module-command)
-    (ghc-read-lisp-list
-     (lambda ()
-       (message "Loading names...")
-       (apply 'call-process ghc-module-command nil '(t nil) nil
-	      `(,@(ghc-make-ghc-options) "-l" "browse" ,@mods))
-       (message "Loading names...done"))
-     (length mods))))
+  (if mods
+      (mapcar 'ghc-load-module mods)
+    (message "No new modules")
+    nil))
+
+(defun ghc-load-module (mod)
+  (prog2
+      (message "Loading symbols for %s..." mod)
+      (ghc-sync-process (format "browse %s\n" mod))
+    (message "Loading symbols for %s...done" mod)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -218,7 +217,7 @@ unloaded modules are loaded")
 (defun ghc-completion-start-point ()
   (save-excursion
     (let ((beg (save-excursion (beginning-of-line) (point)))
-	  (regex (if (ghc-module-completion-p) "[ (,`]" "[ (,`.]")))
+	  (regex (if (ghc-module-completion-p) "[ (,`]" "[\[ (,`.]")))
       (if (re-search-backward regex beg t)
 	  (1+ (point))
 	beg))))
@@ -230,8 +229,7 @@ unloaded modules are loaded")
 
 (defun ghc-import-module ()
   (interactive)
-  (when (eq major-mode 'haskell-mode)
-    (ghc-load-module-buffer)))
+  (ghc-load-module-buffer))
 
 (defun ghc-unloaded-modules (mods)
   (ghc-filter (lambda (mod)
@@ -239,9 +237,6 @@ unloaded modules are loaded")
 		     (not (member mod ghc-loaded-module))))
 	      mods))
 
-(defun ghc-load-module-all-buffers ()
-  (ghc-load-merge-modules (ghc-gather-import-modules-all-buffers)))
-
 (defun ghc-load-module-buffer ()
   (ghc-load-merge-modules (ghc-gather-import-modules-buffer)))
 
@@ -255,7 +250,7 @@ unloaded modules are loaded")
 (defun ghc-merge-keywords (mods)
   (setq ghc-loaded-module (append mods ghc-loaded-module))
   (let* ((modkeys (mapcar 'ghc-module-keyword ghc-loaded-module))
-	 (keywords (cons ghc-reserved-keyword modkeys))
+	 (keywords (cons ghc-extra-keywords (cons ghc-reserved-keyword modkeys)))
 	 (uniq-sorted (sort (ghc-uniq-lol keywords) 'string<)))
     (setq ghc-merged-keyword uniq-sorted)))
 
@@ -265,24 +260,6 @@ unloaded modules are loaded")
 (defun ghc-module-keyword (mod)
   (symbol-value (ghc-module-symbol mod)))
 
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(ghc-defstruct buffer name file)
-
-(defun ghc-buffer-name-file (buf)
-  (ghc-make-buffer (buffer-name buf) (buffer-file-name buf)))
-
-(defun ghc-gather-import-modules-all-buffers ()
-  (let ((bufs (mapcar 'ghc-buffer-name-file (buffer-list)))
-	ret file)
-    (save-excursion
-      (dolist (buf bufs (ghc-uniq-lol ret))
-	(setq file (ghc-buffer-get-file buf))
-	(when (and file (string-match "\\.hs$" file))
-	  (set-buffer (ghc-buffer-get-name buf))
-	  (ghc-add ret (ghc-gather-import-modules-buffer)))))))
-
 (defun ghc-gather-import-modules-buffer ()
   (let (ret)
     (save-excursion
@@ -297,8 +274,4 @@ unloaded modules are loaded")
 ;;; Background Idle Timer
 ;;;
 
-(defalias 'ghc-idle-timer 'ghc-load-module-all-buffer)
-
-(defun ghc-load-module-all-buffer () nil)
-
 (provide 'ghc-comp)
