@@ -9,6 +9,7 @@
 ;;; Code:
 
 (require 'ghc-func)
+(require 'ghc-process)
 
 (defun ghc-show-info (&optional ask)
   (interactive "P")
@@ -21,10 +22,9 @@
        (lambda () (insert info))))))
 
 (defun ghc-get-info (expr)
-  (let* ((modname (or (ghc-find-module-name) "Main"))
-	 (file (buffer-file-name))
-	 (cmds (list "info" file modname expr)))
-    (ghc-run-ghc-mod cmds)))
+  (let* ((file (buffer-file-name))
+	 (cmd (format "info %s %s\n" file expr)))
+    (ghc-sync-process cmd)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -77,14 +77,8 @@
 
 (defun ghc-show-type ()
   (interactive)
-  (if (not (executable-find ghc-module-command))
-      (message "%s not found" ghc-module-command)
-    (let ((modname (or (ghc-find-module-name) "Main")))
-      (ghc-show-type0 modname))))
-
-(defun ghc-show-type0 (modname)
-  (let* ((buf (current-buffer))
-	 (tinfos (ghc-type-get-tinfos modname)))
+  (let ((buf (current-buffer))
+	(tinfos (ghc-type-get-tinfos)))
     (if (null tinfos)
 	(progn
 	  (ghc-type-clear-overlay)
@@ -100,28 +94,30 @@
 	(move-overlay ghc-type-overlay (- left 1) (- right 1) buf)
 	(message type)))))
 
-(defun ghc-type-get-tinfos (modname)
+(defun ghc-type-get-tinfos ()
   (if (= (ghc-type-get-point) (point))
       (ghc-type-set-ix
        (mod (1+ (ghc-type-get-ix)) (length (ghc-type-get-types))))
-    (ghc-type-set-types (ghc-type-obtain-tinfos modname))
-    (ghc-type-set-point (point))
-    (ghc-type-set-ix 0))
+    (let ((types (ghc-type-obtain-tinfos)))
+      (if (not (listp types)) ;; main does not exist in Main
+	  (ghc-type-set-types nil)
+	(ghc-type-set-types types)
+	(ghc-type-set-point (point))
+	(ghc-type-set-ix 0))))
   (ghc-type-get-types))
 
-(defun ghc-type-obtain-tinfos (modname)
+(defun ghc-type-obtain-tinfos ()
   (let* ((ln (int-to-string (line-number-at-pos)))
 	 (cn (int-to-string (1+ (current-column))))
-	 (cdir default-directory)
-	 (file (buffer-file-name)))
-    (ghc-read-lisp
-     (lambda ()
-       (cd cdir)
-       (apply 'call-process ghc-module-command nil t nil
-	      `(,@(ghc-make-ghc-options) "-l" "type" ,file ,modname ,ln ,cn))
-       (goto-char (point-min))
-       (while (search-forward "[Char]" nil t)
-	 (replace-match "String"))))))
+	 (file (buffer-file-name))
+	 (cmd (format "type %s %s %s\n" file ln cn)))
+    (ghc-sync-process cmd nil 'ghc-type-fix-string)))
+
+(defun ghc-type-fix-string ()
+  (save-excursion
+    (goto-char (point-min))
+    (while (search-forward "[Char]" nil t)
+      (replace-match "String"))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -131,7 +127,7 @@
 (defun ghc-expand-th ()
   (interactive)
   (let* ((file (buffer-file-name))
-	 (cmds (list "expand" file))
+	 (cmds (list "expand" file "-b" "\n"))
 	 (source (ghc-run-ghc-mod cmds)))
     (when source
       (ghc-display
@@ -146,7 +142,8 @@
 (defun ghc-get-pos (buf line col)
   (save-excursion
     (set-buffer buf)
-    (goto-line line)
+    (goto-char (point-min))
+    (forward-line (1- line))
     (forward-char col)
     (point)))
 
