@@ -7,13 +7,28 @@
 ;; Put the following code to your "~/.emacs".
 ;;
 ;; (autoload 'ghc-init "ghc" nil t)
+;; (autoload 'ghc-debug "ghc" nil t)
 ;; (add-hook 'haskell-mode-hook (lambda () (ghc-init)))
-;; Or
-;; (add-hook 'haskell-mode-hook (lambda () (ghc-init) (flymake-mode)))
+;;
+;; Or if you wish to display error each goto next/prev error,
+;; set ghc-display-error valiable.
+;;
+;; (setq ghc-display-error 'minibuffer) ; to minibuffer
+;; ; (setq ghc-display-error 'other-buffer) ; to other-buffer
+
+;;
 
 ;;; Code:
 
-(defconst ghc-version "2.0.0")
+;; defvar-local was introduced in 24.3
+(let* ((major 24)
+       (minor 3))
+  (if (or (< emacs-major-version major)
+	  (and (= emacs-major-version major)
+	       (< emacs-minor-version minor)))
+      (error "ghc-mod requires at least Emacs %d.%d" major minor)))
+
+(defconst ghc-version "4.1.1")
 
 ;; (eval-when-compile
 ;;  (require 'haskell-mode))
@@ -21,7 +36,7 @@
 (require 'ghc-comp)
 (require 'ghc-doc)
 (require 'ghc-info)
-(require 'ghc-flymake)
+(require 'ghc-check)
 (require 'ghc-command)
 (require 'ghc-ins-mod)
 (require 'ghc-indent)
@@ -50,9 +65,10 @@
 (defvar ghc-info-key        "\C-c\C-i")
 (defvar ghc-check-key       "\C-x\C-s")
 (defvar ghc-toggle-key      "\C-c\C-c")
+(defvar ghc-jump-key        "\C-c\C-j")
 (defvar ghc-module-key      "\C-c\C-m")
 (defvar ghc-expand-key      "\C-c\C-e")
-(defvar ghc-jump-key        "\C-c\C-j")
+(defvar ghc-kill-key        "\C-c\C-k")
 (defvar ghc-hoogle-key      (format "\C-c%c" (ghc-find-C-h)))
 (defvar ghc-shallower-key   "\C-c<")
 (defvar ghc-deeper-key      "\C-c>")
@@ -74,23 +90,50 @@
     (define-key haskell-mode-map ghc-type-key        'ghc-show-type)
     (define-key haskell-mode-map ghc-info-key        'ghc-show-info)
     (define-key haskell-mode-map ghc-expand-key      'ghc-expand-th)
-    (define-key haskell-mode-map ghc-jump-key        'ghc-flymake-jump)
     (define-key haskell-mode-map ghc-import-key      'ghc-import-module)
-    (define-key haskell-mode-map ghc-previous-key    'flymake-goto-prev-error)
-    (define-key haskell-mode-map ghc-next-key        'flymake-goto-next-error)
-    (define-key haskell-mode-map ghc-help-key        'ghc-flymake-display-errors)
+    (define-key haskell-mode-map ghc-previous-key    'ghc-goto-prev-error)
+    (define-key haskell-mode-map ghc-next-key        'ghc-goto-next-error)
+    (define-key haskell-mode-map ghc-help-key        'ghc-display-errors)
     (define-key haskell-mode-map ghc-insert-key      'ghc-insert-template)
     (define-key haskell-mode-map ghc-sort-key        'ghc-sort-lines)
     (define-key haskell-mode-map ghc-check-key       'ghc-save-buffer)
-    (define-key haskell-mode-map ghc-toggle-key      'ghc-flymake-toggle-command)
+    (define-key haskell-mode-map ghc-toggle-key      'ghc-toggle-check-command)
+    (define-key haskell-mode-map ghc-jump-key        'ghc-jump-file)
     (define-key haskell-mode-map ghc-module-key      'ghc-insert-module)
+    (define-key haskell-mode-map ghc-kill-key        'ghc-kill-process)
     (define-key haskell-mode-map ghc-hoogle-key      'haskell-hoogle)
     (define-key haskell-mode-map ghc-shallower-key   'ghc-make-indent-shallower)
     (define-key haskell-mode-map ghc-deeper-key      'ghc-make-indent-deeper)
     (ghc-comp-init)
-    (setq ghc-initialized t)))
+    (setq ghc-initialized t))
+  (ghc-import-module)
+  (ghc-check-syntax))
 
 (defun ghc-abbrev-init ()
   (set (make-local-variable 'dabbrev-case-fold-search) nil))
 
+;;;###autoload
+(defun ghc-debug ()
+  (interactive)
+  (let ((el-path (locate-file "ghc.el" load-path))
+	(ghc-path (executable-find "ghc"))
+	(ghc-mod-path (executable-find ghc-module-command))
+	(ghc-modi-path (executable-find ghc-interactive-command))
+	(el-ver ghc-version)
+	(ghc-ver (ghc-run-ghc-mod '("--version") "ghc"))
+	(ghc-mod-ver (ghc-run-ghc-mod '("version")))
+	(ghc-modi-ver (ghc-run-ghc-mod '("version") ghc-interactive-command)))
+    (switch-to-buffer (get-buffer-create "**GHC Debug**"))
+    (erase-buffer)
+    (insert "Path: check if you are using intended programs.\n")
+    (insert (format "\t  ghc.el path: %s\n" el-path))
+    (insert (format "\t ghc-mod path: %s\n" ghc-mod-path))
+    (insert (format "\tghc-modi path: %s\n" ghc-modi-path))
+    (insert (format "\t     ghc path: %s\n" ghc-path))
+    (insert "\nVersion: all versions must be the same.\n")
+    (insert (format "\t  ghc.el version %s\n" el-ver))
+    (insert (format "\t %s\n" ghc-mod-ver))
+    (insert (format "\t%s\n" ghc-modi-ver))
+    (insert (format "\t%s\n" ghc-ver))))
+
 (provide 'ghc)
