@@ -8,5 +8,6 @@ spec :: Spec
 spec = do
     describe "listModules" $ do
         it "lists up module names" $ do
-            modules <- lines <$> listModules defaultOptions
+            cradle <- findCradle
+            modules <- lines <$> listModules defaultOptions cradle
             modules `shouldContain` ["Data.Map"]
